declare type EventType = 'enter' | 'exit';
declare type ElementFn = (el: Element, attr: string) => void;
/**
 * An instance of this class tracks entrances and exits
 * in the DOM of elements with a given attribute name
 * and allows an observer to know when those events happen
 * along with the attribute value corresponding to that
 * name.
 *
 * Note that any enter/exit listeners added after the
 * start() method is called will potentially miss some
 * events, particularly for any attributes that exist on
 * or under root when start is called. To avoid this, add
 * listeners first and then call start():
 *
 *   ElementLifecycle.of(root)
 *     .on('enter', () => { ... })
 *     .on('exit', () => { ... })
 *     .start();
 */
export declare class ElementLifecycle {
    static of(root: Element, attr: string): ElementLifecycle;
    private mode;
    private readonly root;
    private readonly attribute;
    private readonly attributeSelector;
    private constructor();
    on(type: EventType, handler: ElementFn): ElementLifecycle;
    start(): ElementLifecycle;
    stop(): void;
    private enter;
    private exit;
    /** Traverse the subtree of an added or removed
     * node looking for vacancies. Ensure duplicates
     * aren't reported using the given dedupe set. */
    private subtree;
}
export {};
//# sourceMappingURL=element-lifecycle.d.ts.map