"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GateKeeper = void 0;
const NOOP = () => { };
/**
 * Tracks appearances and disappearances of strings within a set.
 */
class GateKeeper {
    constructor() {
        this.map = new Map();
    }
    /**
     * Add an instance of the given string. If there are already some
     * instances of this string, this returns undefined. Otherwise it
     * returns a promise which is resolved when the last instance of
     * that string leaves.
     */
    incr(str) {
        const state = this.map.get(str);
        if (state !== undefined) {
            this.map.set(str, Object.assign(Object.assign({}, state), { count: state.count + 1 }));
            return undefined;
        }
        else {
            const newState = {
                count: 1,
                resolvable: new ExternallyResolvable(),
            };
            this.map.set(str, newState);
            return newState.resolvable.promise;
        }
    }
    /**
     * Remove an instance of the given string. If this is the last one
     * to leave, it will internally trigger a resolution to the promise
     * created when the first instance arriveds.
     */
    decr(str) {
        const state = this.map.get(str);
        if (state !== undefined) {
            const newCount = state.count - 1;
            if (newCount < 1) {
                this.map.delete(str);
                state.resolvable.resolve();
            }
            else {
                this.map.set(str, Object.assign(Object.assign({}, state), { count: newCount }));
            }
        }
    }
}
exports.GateKeeper = GateKeeper;
/**
 * This is a wrapper on a Promise object that can be resolved from
 * the outside.
 */
class ExternallyResolvable {
    constructor() {
        this.resolver = NOOP;
        this.promise = new Promise(resolver => {
            this.resolver = resolver;
        });
    }
    resolve() {
        this.resolver();
    }
}
